/* MkRelax, Visual Relax Editor
 *  Copyright (C) 2001-2002 SAKURAI, Masashi (m.sakurai@cmt.phys.kyushu-u.ac.jp)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package jp.gr.java_conf.ccs2.tool.mkrelax;
import java.util.*;
import java.io.Serializable;

public class HedgeRuleElementModel implements Serializable,ElementOperation {
    static final long serialVersionUID = 46000408259074347L;

    String label,comment;

    AbstractHedgeModel model ;
    ModuleModel parentModule;

    public HedgeRuleElementModel (ModuleModel mm,String r) {
	parentModule = mm;
	label = r;
    }

    public ModuleModel getParentModule() {return parentModule;}

    public void setLabel(String a) { label = a; }
    public String getLabel() { return label; }

    public void setComment(String a) { comment = a; }
    public String getComment() { return comment; }

    public void setElement(AbstractHedgeModel a) { model = a; }
    public AbstractHedgeModel getElement() { return model; }


    // ElementOperation stuff

    public boolean canAddElement() {
	return (model==null)?true:false;
    }

    public void addOpElements(ElementOperation [] args) {
	if (args == null || args.length==0) return;
	if (acceptOpElement(args[0])) {
	    if (args[0] instanceof AbstractHedgeModel) {
		setElement((AbstractHedgeModel)args[0]);
	    } else if (args[0] instanceof ObjectModel) {
		setElement(ElementUtil.getRef(getParentModule(),
					      (ObjectModel)args[0]));
	    } else {
		throw new InternalError("Object is now the subclass of ElementModel.(bug)");
	    }
	}
    }

    public ElementOperation [] getOpElements(){
	if (getElement() == null) return null;
	ElementOperation [] ret = {getElement()};
	return ret;
    }

    public void removeOpElement(ElementOperation arg) {
	if (arg == getElement()) {
	    setElement(null);
	    return;
	} else if (getElement() instanceof RefElementModel) {
	    RefElementModel ref = (RefElementModel)getElement();
	    if (ref.getAt() == arg) {
		setElement(null);
		return;
	    }
	}
	throw new InternalError("Invalid argument.");
    }

    public boolean acceptOpElement(ElementOperation e) {
	if (e.getOpElementLevel() == HEDGE ||
	    e.getOpElementLevel() == OBJECT)
	    return true;
	return false;
    }

    public int getOpElementLevel() {
	return -1;
    }

    public ElementOperation getOpCopy() {
	HedgeRuleElementModel ret = ElementUtil.getHedgeRule(getParentModule(),
							     getLabel());
	if (getOpElements() != null) {
	    ret.addOpElements
		(ElementUtil.toArray(getOpElements()[0].getOpCopy()));
	}
	return ret;
    }
}
